package SPVM::Eg::Node::ShadowRoot;



1;

=head1 Name

SPVM::Eg::Node::ShadowRoot - ShadowRoot in JavaScript

=head1 Description

The Eg::Node::ShadowRoot class in L<SPVM> represents the root node of a DOM subtree that is rendered separately from a document's main DOM tree.

This class is a port of L<ShadowRoot|https://developer.mozilla.org/en-US/docs/Web/API/ShadowRoot> in JavaScript.

=head1 Inheritance

L<Eg::Node::DocumentFragment|SPVM::Eg::Node::DocumentFragment>

=head1 Copyright & License

Copyright (c) 2024 Yuki Kimoto

MIT License
