package SPVM::Eg::Node::Element::HTML::Html;



1;

=head1 Name

SPVM::Eg::Node::Element::HTML::Html - HTMLHtmlElement in JavaScript

=head1 Description

The Eg::Node::Element::HTML::Html class in L<SPVM> represents L<HTMLHtmlElement|https://developer.mozilla.org/en-US/docs/Web/API/HTMLHtmlElement> in JavaScript

=head1 Copyright & License

Copyright (c) 2024 Yuki Kimoto

MIT License

