package SPVM::Eg::Node::Element::HTML::Canvas;



1;

=head1 Name

SPVM::Eg::Node::Element::HTML::Canvas - HTMLCanvasElement in JavaScript

=head1 Description

The Eg::Node::Element::HTML::Canvas class in L<SPVM> represents L<HTMLCanvasElement|https://developer.mozilla.org/en-US/docs/Web/API/HTMLCanvasElement> in JavaScript

=head1 Copyright & License

Copyright (c) 2024 Yuki Kimoto

MIT License

