package SPVM::Eg::Node::DocumentFragment;



1;

=head1 Name

SPVM::Eg::Node::DocumentFragment - DocumentFragment in JavaScript

=head1 Description

The Eg::Node::DocumentFragment class in L<SPVM> represents a minimal document object that has no parent.

This class is a port of L<DocumentFragment|https://developer.mozilla.org/en-US/docs/Web/API/DocumentFragment> in JavaScript.

=head1 Inheritance

L<Eg::Node|SPVM::Eg::Node>

=head1 Copyright & License

Copyright (c) 2024 Yuki Kimoto

MIT License
