package SPVM::Eg::Node::Element::HTML::Details;



1;

=head1 Name

SPVM::Eg::Node::Element::HTML::Details - HTMLDetailsElement in JavaScript

=head1 Description

The Eg::Node::Element::HTML::Details class in L<SPVM> represents L<HTMLDetailsElement|https://developer.mozilla.org/en-US/docs/Web/API/HTMLDetailsElement> in JavaScript

=head1 Copyright & License

Copyright (c) 2024 Yuki Kimoto

MIT License

